from claim.apps import ClaimConfig
from core.schema import (
    OrderedDjangoFilterConnectionField,
    signal_mutation_module_validate,
)
import graphene
from django.core.exceptions import PermissionDenied
from django.db.models import Prefetch
from django.db.models import Q
from .services import ByInsureeRequest, ByInsureeService
from .services import ByFamilyRequest, ByFamilyService
from .services import EligibilityRequest, EligibilityService
from .apps import PolicyConfig
from django.utils.translation import gettext as _
import graphene_django_optimizer as gql_optimizer
from graphene_django.filter import DjangoFilterConnectionField
from core.models import Officer
from .models import PolicyMutation
from product.models import Product
from contribution.models import Premium
from insuree.models import Family, Insuree, InsureePolicy
from django.db.models import OuterRef, Subquery, Sum, F, Count
from location.apps import LocationConfig

# We do need all queries and mutations in the namespace here.
from .gql_queries import *  # lgtm [py/polluting-import]
from .gql_mutations import *  # lgtm [py/polluting-import]

from .values import policy_values


class Query(graphene.ObjectType):
    policy_values = graphene.Field(
        PolicyAndWarningsGQLType,
        prev_uuid=graphene.String(required=False),
        stage=graphene.String(required=True),
        enrollDate=graphene.DateTime(required=True),
        product_id=graphene.Int(required=True),
        family_id=graphene.Int(required=True)
    )
    policies = OrderedDjangoFilterConnectionField(
        PolicyGQLType,
        region_id=graphene.Int(),
        district_id=graphene.Int(),
        balance_lte=graphene.Float(),
        balance_gte=graphene.Float(),
        showHistory=graphene.Boolean(),
        showInactive=graphene.Boolean(),
        confirmationType=graphene.String(),
        orderBy=graphene.List(of_type=graphene.String),
    )
    # Note:
    # A Policy is bound to a Family...
    # but an insuree of the family is only covered by the family policy
    # if there is a (valid) InsureePolicy
    policies_by_insuree = graphene.relay.ConnectionField(
        PolicyByFamilyOrInsureeConnection,
        chf_id=graphene.String(required=True),
        active_or_last_expired_only=graphene.Boolean(),
        show_history=graphene.Boolean(),
        order_by=graphene.String(),
        target_date=graphene.Date(),
    )
    policies_by_family = graphene.relay.ConnectionField(
        PolicyByFamilyOrInsureeConnection,
        family_uuid=graphene.String(required=True),
        active_or_last_expired_only=graphene.Boolean(),
        show_history=graphene.Boolean(),
        order_by=graphene.String(),
        target_date=graphene.Date(),
    )
    # TODO: refactoring
    # Eligibility is calculated for a Policy... which is bound to a Family (not an Insuree)
    # YET: family member may not be covered by the policy!!
    # This requires to refactor the EligibilityService
    policy_eligibility_by_insuree = graphene.Field(
        EligibilityGQLType,
        chfId=graphene.String(required=True)
    )
    policy_item_eligibility_by_insuree = graphene.Field(
        EligibilityGQLType,
        chfId=graphene.String(required=True),
        itemCode=graphene.String(required=True)
    )
    policy_service_eligibility_by_insuree = graphene.Field(
        EligibilityGQLType,
        chfId=graphene.String(required=True),
        serviceCode=graphene.String(required=True),
    )
    policy_officers = DjangoFilterConnectionField(
        OfficerGQLType,
        search=graphene.String(),
        district=graphene.String(),
        region=graphene.String(),
    )

    def resolve_policy_values(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_policies_perms):
            raise PermissionDenied(_("unauthorized"))

        product = Product.objects.filter(
            Q(validity_to__isnull=True),
            Q(id=kwargs.get('product_id')) | Q(legacy_id=kwargs.get('product_id')),
            Q(validity_from__date__lte=kwargs.get('enrollDate')) | Q(date_from__lte=kwargs.get('enrollDate')),
        ).order_by('-validity_from').first()

        if not product:
            raise ValidationError('Provided product not available')

        policy = PolicyGQLType(
            stage=kwargs.get('stage'),
            enroll_date=kwargs.get('enrollDate'),
            start_date=kwargs.get('enrollDate'),
            product=product,
        )
        prefetch = Prefetch(
            'members',
            queryset=Insuree.objects.filter(
                validity_to__isnull=True).order_by('validity_from')
        )
        family = Family.objects \
            .prefetch_related(prefetch) \
            .get(id=kwargs.get('family_id'))
        prev_policy = None
        if 'prev_uuid' in kwargs:
            prev_policy = Policy.objects.get(uuid=kwargs.get('prev_uuid'))
        policy, warnings = policy_values(policy, family, prev_policy)
        return PolicyAndWarningsGQLType(policy=policy, warnings=warnings)

    def resolve_policies(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_policies_perms):
            raise PermissionDenied(_("unauthorized"))
        query = Policy.objects
        if not kwargs.get('showHistory', False):
            query = query.filter(*filter_validity(**kwargs))
        if kwargs.get('showInactive', False):
            family_count = Insuree.objects.values('family_id')\
                .filter(validity_to__isnull=True).annotate(m_count=Count('id'))
            family_sq = family_count.filter(family_id=OuterRef('family_id'))
            covered_count = InsureePolicy.objects.values('policy_id')\
                .filter(validity_to__isnull=True).annotate(i_count=Count('id'))
            covered_sq = covered_count.filter(policy_id=OuterRef('id'))
            query = query.annotate(
                inactive_count=Subquery(family_sq.values(
                    'm_count')) - Subquery(covered_sq.values('i_count'))
            )
            query = query.filter(inactive_count__gt=0)
        if kwargs.get('balance_lte') or kwargs.get('balance_gte') or kwargs.get('sum_premiums', False):
            query=query.annotate(
                sum_premiums=Policy.get_query_sum_premium()
                )
        if kwargs.get('balance_lte') or kwargs.get('balance_gte'):
            query = query.annotate(
                balance=F('value') - F('sum_premiums'))
        if kwargs.get('balance_lte'):
            query = query.filter(balance__lte=kwargs.get('balance_lte'))
        if kwargs.get('balance_gte'):
            query = query.filter(balance__gte=kwargs.get('balance_gte'))
        if kwargs.get('confirmationType'):
            query = query.filter(family__confirmation_type=kwargs.get('confirmationType'))
        location_id = kwargs.get('district_id') if kwargs.get(
            'district_id') else kwargs.get('region_id')
        if location_id:
            location_level = 2 if kwargs.get('district_id') else 1
            f = '_id'
            for i in range(len(LocationConfig.location_types) - location_level):
                f = "__parent" + f
            f = PolicyConfig.policy_location_via + '__location' + f
            query = query.filter(Q(**{f: location_id}))
        return gql_optimizer.query(query.all(), info)

    @staticmethod
    def _to_policy_by_family_or_insuree_item(item):
        return PolicyByFamilyOrInsureeGQLType(
            policy_id=item.policy_id,
            policy_uuid=item.policy_uuid,
            policy_value=item.policy_value,
            product_code=item.product_code,
            product_name=item.product_name,
            start_date=item.start_date,
            enroll_date=item.enroll_date,
            effective_date=item.effective_date,
            expiry_date=item.expiry_date,
            officer_code=item.officer_code,
            officer_name=item.officer_name,
            status=item.status,
            ded=item.ded,
            ded_in_patient=item.ded_in_patient,
            ded_out_patient=item.ded_out_patient,
            ceiling=item.ceiling,
            ceiling_in_patient=item.ceiling_in_patient,
            ceiling_out_patient=item.ceiling_out_patient,
            balance=item.balance,
            validity_from=item.validity_from,
            validity_to=item.validity_to,
            max_installments=item.max_installments,
        )

    def resolve_policies_by_insuree(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_policies_by_insuree_perms) \
                and not info.context.user.has_perms(ClaimConfig.gql_query_claims_perms):
            raise PermissionDenied(_("unauthorized"))
        req = ByInsureeRequest(
            chf_id=kwargs.get('chf_id'),
            active_or_last_expired_only=kwargs.get(
                'active_or_last_expired_only', False),
            show_history=kwargs.get('show_history', False),
            order_by=kwargs.get('order_by', None),
            target_date=kwargs.get('target_date', None)
        )
        res = ByInsureeService(user=info.context.user).request(req)
        return [Query._to_policy_by_family_or_insuree_item(x) for x in res.items]

    def resolve_policies_by_family(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_policies_by_family_perms):
            raise PermissionDenied(_("unauthorized"))
        req = ByFamilyRequest(
            family_uuid=kwargs.get('family_uuid'),
            active_or_last_expired_only=kwargs.get(
                'active_or_last_expired_only', False),
            show_history=kwargs.get('show_history', False),
            order_by=kwargs.get('order_by', None),
            target_date=kwargs.get('target_date', None)
        )
        res = ByFamilyService(user=info.context.user).request(req)
        return [Query._to_policy_by_family_or_insuree_item(x) for x in res.items]

    @staticmethod
    def _resolve_policy_eligibility_by_insuree(user, req):
        res = EligibilityService(user=user).request(req)
        return EligibilityGQLType(
            prod_id=res.prod_id,
            total_admissions_left=res.total_admissions_left,
            total_visits_left=res.total_visits_left,
            total_consultations_left=res.total_consultations_left,
            total_surgeries_left=res.total_surgeries_left,
            total_deliveries_left=res.total_deliveries_left,
            total_antenatal_left=res.total_antenatal_left,
            consultation_amount_left=res.consultation_amount_left,
            surgery_amount_left=res.surgery_amount_left,
            delivery_amount_left=res.delivery_amount_left,
            hospitalization_amount_left=res.hospitalization_amount_left,
            antenatal_amount_left=res.antenatal_amount_left,
            min_date_service=res.min_date_service,
            min_date_item=res.min_date_item,
            service_left=res.service_left,
            item_left=res.item_left,
            is_item_ok=res.is_item_ok,
            is_service_ok=res.is_service_ok
        )

    def resolve_policy_eligibility_by_insuree(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_eligibilities_perms):
            raise PermissionDenied(_("unauthorized"))
        req = EligibilityRequest(
            chf_id=kwargs.get('chfId')
        )
        return Query._resolve_policy_eligibility_by_insuree(
            user=info.context.user,
            req=req
        )

    def resolve_policy_item_eligibility_by_insuree(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_eligibilities_perms):
            raise PermissionDenied(_("unauthorized"))
        req = EligibilityRequest(
            chf_id=kwargs.get('chfId'),
            item_code=kwargs.get('itemCode')
        )
        return Query._resolve_policy_eligibility_by_insuree(
            user=info.context.user,
            req=req
        )

    def resolve_policy_service_eligibility_by_insuree(self, info, **kwargs):
        if not info.context.user.has_perms(PolicyConfig.gql_query_eligibilities_perms):
            raise PermissionDenied(_("unauthorized"))
        req = EligibilityRequest(
            chf_id=kwargs.get('chfId'),
            service_code=kwargs.get('serviceCode')
        )
        return Query._resolve_policy_eligibility_by_insuree(
            user=info.context.user,
            req=req
        )

    def resolve_policy_officers(
            self,
            info,
            search=None,
            district=None,
            region=None,
            **kwargs
    ):
        if not info.context.user.has_perms(
            PolicyConfig.gql_query_policy_officers_perms
        ):
            raise PermissionDenied(_("unauthorized"))
        queryset = Officer.objects
        location_id = district if district else region

        if location_id is not None:
            location = int(location_id)
            queryset = queryset.filter(
                Q(officer_villages__location__isnull=False,
                  officer_villages__location__id=location)  # villages
                | Q(officer_villages__location__parent_id__isnull=False,
                    officer_villages__location__parent_id=location)  # municipalities
                | Q(officer_villages__location__parent__parent_id__isnull=False,
                    officer_villages__location__parent__parent_id=location)  # districts
                | Q(officer_villages__location__parent__parent__parent_id__isnull=False,
                    officer_villages__location__parent__parent__parent_id=location)  # regions
            ).distinct()

        if search is not None:
            queryset = queryset.filter(
                Q(code__icontains=search)
                | Q(last_name__icontains=search)
                | Q(other_names__icontains=search)
            )

        return queryset


class Mutation(graphene.ObjectType):
    create_policy = CreatePolicyMutation.Field()
    update_policy = UpdatePolicyMutation.Field()
    delete_policies = DeletePoliciesMutation.Field()
    renew_policy = RenewPolicyMutation.Field()
    suspend_policies = SuspendPoliciesMutation.Field()


def on_policy_mutation(sender, **kwargs):
    uuids = kwargs['data'].get('uuids', [])
    if not uuids:
        uuid = kwargs['data'].get('policy_uuid', None)
        uuids = [uuid] if uuid else []
    if not uuids:
        return []
    impacted_policies = Policy.objects.filter(uuid__in=uuids).all()
    for policy in impacted_policies:
        PolicyMutation.objects.create(
            policy=policy, mutation_id=kwargs['mutation_log_id'])
    return []


def bind_signals():
    signal_mutation_module_validate["policy"].connect(on_policy_mutation)



from django.db.models import Func, DateTimeField
from django.db.models.functions import Cast, Coalesce


class MonthsAdd(Func):
    """
    Custom function that is suitable for MS SQL and Postgresql. If using different database it is possible that
    it might need an update with custom resolve.
    Usage: Foo.objects.annotate(end_date=MonthsAdd('start_date', 'duration')).filter(end_date__gt=datetime.now)
    """
    # https://stackoverflow.com/questions/33981468/using-dateadd-in-django-filter

    arg_joiner = " + CAST("
    template = "%(expressions)s || 'months' as INTERVAL)"

    template_mssql = '%(function)s(MONTH, %(expressions)s)'
    function_mssql = "DATEADD"

    output_field = DateTimeField()
    arity = 2
    COUNTER = 0

    def as_sql(self, compiler, connection, **extra_context):

        if connection.vendor == 'microsoft':
            self.arg_joiner = ', '
            self.source_expressions = self.get_source_expressions(
            )[::-1] if self.COUNTER == 0 else self.get_source_expressions()
            self.template = self.template_mssql
            self.function = self.function_mssql
        self.COUNTER += 1
        return super().as_sql(compiler, connection, **extra_context)


def get_queryset_valid_at_date(queryset, date):
    filtered_qs = queryset.filter(
        validity_to__gte=date,
        validity_from__lte=date
    )
    if len(filtered_qs) > 0:
        return filtered_qs
    return queryset.filter(validity_from__date__lte=date, validity_to__isnull=True)



# Generated by Django 3.2.19 on 2023-07-12 17:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('policy', '0006_set_model_managed_to_true'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='policymutation',
            table='policy_PolicyMutation',
        ),
    ]



from core.test_helpers import create_test_interactive_user
from rest_framework import status
from rest_framework.test import APITestCase
from dataclasses import dataclass
from graphql_jwt.shortcuts import get_token
from core.models import User
from django.conf import settings


@dataclass
class DummyContext:
    """ Just because we need a context to generate. """
    user: User


class ReportAPITests( APITestCase):

    admin_user = None
    admin_token = None
    POI_URL = f'/{settings.SITE_ROOT()}report/policy_primary_operational_indicators/pdf/?yearMonth=2019-04-01'
    PR_URL = f'/{settings.SITE_ROOT()}report/policy_renewals/pdf/?date_start=2019-04-01&date_end=2019-04-30'
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        cls.admin_user = create_test_interactive_user(username="testLocationAdmin")
        cls.admin_token = get_token(cls.admin_user, DummyContext(user=cls.admin_user))
        
    def test_primary_operational_indicators_report(self):
        headers={"HTTP_AUTHORIZATION": f"Bearer {self.admin_token}"}
        response = self.client.get(self.POI_URL, format='json', **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)
        
    def test_policy_renewal_report(self):
        headers={"HTTP_AUTHORIZATION": f"Bearer {self.admin_token}"}
        response = self.client.get(self.PR_URL, format='json', **headers)
        self.assertEqual(response.status_code, status.HTTP_200_OK)


# Create your tests here.

# Create your tests here.
from django.test import TestCase

from contribution.test_helpers import create_test_premium
from core.test_helpers import create_test_officer
from policy.models import Policy
from policy.test_helpers import create_test_policy2,create_test_policy_with_IPs,create_test_insuree_for_policy
from product.test_helpers import create_test_product, create_test_product_service


class TaskGroupServiceTest(TestCase):
    def test_helper(self):
        insuree, family = create_test_insuree_for_policy(with_family=True, is_head=False, custom_props={"chf_id": "paysimp"}, family_custom_props=None)
        product = create_test_product("ELI1")
        (policy, insuree_policy) = create_test_policy2(product, insuree, custom_props={
            "value": 1000, "status": Policy.STATUS_IDLE})

        policy = create_test_policy_with_IPs(product, insuree, valid=True, policy_props={"value": 1000, "status": Policy.STATUS_ACTIVE}, IP_props=None, premium_props=None)



from datetime import datetime, timedelta

from dateutil.relativedelta import relativedelta
from django.conf import settings
from django.db import connection

from tools.utils import dictfetchall
import logging
logger = logging.getLogger(__name__)

# If manually pasting from ReportBro and you have test data, search and replace \" with \\"
template = """{
  "docElements": [
    {
      "elementType": "text",
      "id": 3,
      "containerId": "0_header",
      "x": 120,
      "y": 0,
      "width": 575,
      "height": 40,
      "content": "Policy Primary Operational Indicators",
      "richText": false,
      "richTextContent": null,
      "richTextHtml": "",
      "eval": false,
      "styleId": "",
      "bold": true,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "center",
      "verticalAlignment": "middle",
      "textColor": "#006273",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": "24",
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": 1,
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": 2,
      "paddingTop": 2,
      "paddingRight": 2,
      "paddingBottom": 2,
      "printIf": "",
      "removeEmptyElement": false,
      "alwaysPrintOnSamePage": true,
      "pattern": "",
      "link": "",
      "cs_condition": "",
      "cs_styleId": "",
      "cs_bold": false,
      "cs_italic": false,
      "cs_underline": false,
      "cs_strikethrough": false,
      "cs_horizontalAlignment": "left",
      "cs_verticalAlignment": "top",
      "cs_textColor": "#000000",
      "cs_backgroundColor": "",
      "cs_font": "helvetica",
      "cs_fontSize": 12,
      "cs_lineSpacing": 1,
      "cs_borderColor": "#000000",
      "cs_borderWidth": "1",
      "cs_borderAll": false,
      "cs_borderLeft": false,
      "cs_borderTop": false,
      "cs_borderRight": false,
      "cs_borderBottom": false,
      "cs_paddingLeft": 2,
      "cs_paddingTop": 2,
      "cs_paddingRight": 2,
      "cs_paddingBottom": 2,
      "spreadsheet_hide": false,
      "spreadsheet_column": "",
      "spreadsheet_colspan": "",
      "spreadsheet_addEmptyRow": false,
      "spreadsheet_textWrap": false
    },
    {
      "elementType": "line",
      "id": 158,
      "containerId": "0_content",
      "x": 0,
      "y": 0,
      "width": 820,
      "height": 1,
      "color": "#000000",
      "printIf": ""
    },
    {
      "elementType": "text",
      "id": 193,
      "containerId": "0_content",
      "x": 0,
      "y": 10,
      "width": 575,
      "height": 20,
      "content": "Date: ${yearMonth}",
      "richText": false,
      "richTextContent": null,
      "richTextHtml": "",
      "eval": false,
      "styleId": "",
      "bold": true,
      "italic": false,
      "underline": true,
      "strikethrough": false,
      "horizontalAlignment": "left",
      "verticalAlignment": "top",
      "textColor": "#000000",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": 12,
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": 1,
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": 2,
      "paddingTop": 2,
      "paddingRight": 2,
      "paddingBottom": 2,
      "printIf": "",
      "removeEmptyElement": false,
      "alwaysPrintOnSamePage": true,
      "pattern": "",
      "link": "",
      "cs_condition": "",
      "cs_styleId": "",
      "cs_bold": false,
      "cs_italic": false,
      "cs_underline": false,
      "cs_strikethrough": false,
      "cs_horizontalAlignment": "left",
      "cs_verticalAlignment": "top",
      "cs_textColor": "#000000",
      "cs_backgroundColor": "",
      "cs_font": "helvetica",
      "cs_fontSize": 12,
      "cs_lineSpacing": 1,
      "cs_borderColor": "#000000",
      "cs_borderWidth": "1",
      "cs_borderAll": false,
      "cs_borderLeft": false,
      "cs_borderTop": false,
      "cs_borderRight": false,
      "cs_borderBottom": false,
      "cs_paddingLeft": 2,
      "cs_paddingTop": 2,
      "cs_paddingRight": 2,
      "cs_paddingBottom": 2,
      "spreadsheet_hide": false,
      "spreadsheet_column": "",
      "spreadsheet_colspan": "",
      "spreadsheet_addEmptyRow": false,
      "spreadsheet_textWrap": false
    },
    {
      "elementType": "table",
      "id": 195,
      "containerId": "0_content",
      "width": 745,
      "x": 0,
      "y": 40,
      "dataSource": "${data}",
      "columns": 18,
      "header": true,
      "contentRows": 2,
      "footer": false,
      "border": "grid",
      "borderColor": "#000000",
      "borderWidth": "1",
      "printIf": "",
      "removeEmptyElement": false,
      "spreadsheet_hide": false,
      "spreadsheet_column": "",
      "spreadsheet_addEmptyRow": false,
      "headerData": {
        "elementType": "none",
        "id": 196,
        "height": 36,
        "backgroundColor": "",
        "repeatHeader": true,
        "columnData": [
          {
            "elementType": "table_text",
            "id": 197,
            "width": 84,
            "content": "Product Code",
            "eval": false,
            "colspan": "",
            "styleId": "33",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 205,
            "width": 36,
            "content": "Total Active Insurees",
            "eval": false,
            "colspan": "3",
            "styleId": "",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "center",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 302,
            "width": 36,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 305,
            "width": 36,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 206,
            "width": 35,
            "content": "New insurees",
            "eval": false,
            "colspan": "3",
            "styleId": "",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "center",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 308,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 311,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 239,
            "width": 35,
            "content": "Suspended Insurees",
            "eval": false,
            "colspan": "3",
            "styleId": "",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "center",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 326,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 330,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 283,
            "width": 35,
            "content": "Expired Insurees",
            "eval": false,
            "colspan": "3",
            "styleId": "",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "center",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 334,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 338,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 289,
            "width": 35,
            "content": "Renewed Insurees",
            "eval": false,
            "colspan": "3",
            "styleId": "",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "center",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 342,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 346,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 292,
            "width": 65,
            "content": "Total Premium Paid",
            "eval": false,
            "colspan": "",
            "styleId": "33",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          },
          {
            "elementType": "table_text",
            "id": 352,
            "width": 68,
            "content": "Allocated",
            "eval": false,
            "colspan": "",
            "styleId": "33",
            "bold": true,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "middle",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": "10",
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "printIf": "",
            "growWeight": 0,
            "borderWidth": 1
          }
        ]
      },
      "contentDataRows": [
        {
          "elementType": "none",
          "id": 314,
          "height": 20,
          "backgroundColor": "",
          "alternateBackgroundColor": "",
          "groupExpression": "",
          "printIf": "${row_number}==1",
          "alwaysPrintOnSamePage": true,
          "pageBreak": false,
          "repeatGroupHeader": false,
          "columnData": [
            {
              "elementType": "table_text",
              "id": 315,
              "width": 84,
              "content": "",
              "eval": false,
              "colspan": "",
              "styleId": "33",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 316,
              "width": 36,
              "content": "M",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 317,
              "width": 36,
              "content": "F",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 318,
              "width": 36,
              "content": "O",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 319,
              "width": 35,
              "content": "M",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 320,
              "width": 35,
              "content": "F",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 321,
              "width": 35,
              "content": "O",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 322,
              "width": 35,
              "content": "M",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 327,
              "width": 35,
              "content": "F",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 331,
              "width": 35,
              "content": "O",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 323,
              "width": 35,
              "content": "M",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 335,
              "width": 35,
              "content": "F",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 339,
              "width": 35,
              "content": "O",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 324,
              "width": 35,
              "content": "M",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 343,
              "width": 35,
              "content": "F",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 347,
              "width": 35,
              "content": "O",
              "eval": false,
              "colspan": "",
              "styleId": "350",
              "bold": true,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "center",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "10",
              "lineSpacing": 1,
              "paddingLeft": 0,
              "paddingTop": 0,
              "paddingRight": 0,
              "paddingBottom": 0,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 325,
              "width": 65,
              "content": "",
              "eval": false,
              "colspan": "",
              "styleId": "",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "top",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": 12,
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 353,
              "width": 68,
              "content": "",
              "eval": false,
              "colspan": "",
              "styleId": "",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "top",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": 12,
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            }
          ]
        },
        {
          "elementType": "none",
          "id": 199,
          "height": 20,
          "backgroundColor": "",
          "alternateBackgroundColor": "#eeeeee",
          "groupExpression": "",
          "printIf": "",
          "alwaysPrintOnSamePage": false,
          "pageBreak": false,
          "repeatGroupHeader": false,
          "columnData": [
            {
              "elementType": "table_text",
              "id": 200,
              "width": 84,
              "content": "${ProductCode}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 208,
              "width": 36,
              "content": "${TotalMaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 303,
              "width": 36,
              "content": "${TotalFemaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 306,
              "width": 36,
              "content": "${TotalOtherInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 209,
              "width": 35,
              "content": "${NewMaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 309,
              "width": 35,
              "content": "${NewFemaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 312,
              "width": 35,
              "content": "${NewOtherInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 240,
              "width": 35,
              "content": "${SuspendedMaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 328,
              "width": 35,
              "content": "${SuspendedFemaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 332,
              "width": 35,
              "content": "${SuspendedOtherInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 284,
              "width": 35,
              "content": "${ExpiredMaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 336,
              "width": 35,
              "content": "${ExpiredFemaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 340,
              "width": 35,
              "content": "${ExpiredOtherInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 290,
              "width": 35,
              "content": "${RenewMaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 344,
              "width": 35,
              "content": "${RenewFemaleInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 348,
              "width": 35,
              "content": "${RenewOtherInsurees}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 293,
              "width": 65,
              "content": "${TotalPremiumPaid}",
              "eval": false,
              "colspan": "",
              "styleId": "34",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "middle",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": "9",
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            },
            {
              "elementType": "table_text",
              "id": 354,
              "width": 68,
              "content": "${AvailablePremium}",
              "eval": false,
              "colspan": "",
              "styleId": "",
              "bold": false,
              "italic": false,
              "underline": false,
              "strikethrough": false,
              "horizontalAlignment": "left",
              "verticalAlignment": "top",
              "textColor": "#000000",
              "backgroundColor": "",
              "font": "helvetica",
              "fontSize": 12,
              "lineSpacing": 1,
              "paddingLeft": 2,
              "paddingTop": 2,
              "paddingRight": 2,
              "paddingBottom": 2,
              "pattern": "",
              "link": "",
              "cs_condition": "",
              "cs_styleId": "",
              "cs_bold": false,
              "cs_italic": false,
              "cs_underline": false,
              "cs_strikethrough": false,
              "cs_horizontalAlignment": "left",
              "cs_verticalAlignment": "top",
              "cs_textColor": "#000000",
              "cs_backgroundColor": "",
              "cs_font": "helvetica",
              "cs_fontSize": 12,
              "cs_lineSpacing": 1,
              "cs_paddingLeft": 2,
              "cs_paddingTop": 2,
              "cs_paddingRight": 2,
              "cs_paddingBottom": 2,
              "spreadsheet_textWrap": false,
              "borderWidth": 1,
              "growWeight": 0
            }
          ]
        }
      ],
      "footerData": {
        "elementType": "none",
        "id": 202,
        "height": 20,
        "backgroundColor": "",
        "columnData": [
          {
            "elementType": "table_text",
            "id": 203,
            "width": 84,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 211,
            "width": 36,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 304,
            "width": 36,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 307,
            "width": 36,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 285,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 310,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 313,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 291,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 329,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 333,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 294,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 337,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 341,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 298,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 345,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 349,
            "width": 35,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 301,
            "width": 65,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          },
          {
            "elementType": "table_text",
            "id": 355,
            "width": 68,
            "content": "",
            "eval": false,
            "colspan": "",
            "styleId": "",
            "bold": false,
            "italic": false,
            "underline": false,
            "strikethrough": false,
            "horizontalAlignment": "left",
            "verticalAlignment": "top",
            "textColor": "#000000",
            "backgroundColor": "",
            "font": "helvetica",
            "fontSize": 12,
            "lineSpacing": 1,
            "paddingLeft": 2,
            "paddingTop": 2,
            "paddingRight": 2,
            "paddingBottom": 2,
            "pattern": "",
            "link": "",
            "cs_condition": "",
            "cs_styleId": "",
            "cs_bold": false,
            "cs_italic": false,
            "cs_underline": false,
            "cs_strikethrough": false,
            "cs_horizontalAlignment": "left",
            "cs_verticalAlignment": "top",
            "cs_textColor": "#000000",
            "cs_backgroundColor": "",
            "cs_font": "helvetica",
            "cs_fontSize": 12,
            "cs_lineSpacing": 1,
            "cs_paddingLeft": 2,
            "cs_paddingTop": 2,
            "cs_paddingRight": 2,
            "cs_paddingBottom": 2,
            "spreadsheet_textWrap": false,
            "borderWidth": 1,
            "growWeight": 0
          }
        ]
      }
    },
    {
      "elementType": "text",
      "id": 7,
      "containerId": "0_footer",
      "x": 320,
      "y": 0,
      "width": 255,
      "height": 30,
      "content": "Page ${page_number} / ${page_count}",
      "richText": false,
      "richTextContent": null,
      "richTextHtml": "",
      "eval": false,
      "styleId": "",
      "bold": false,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "right",
      "verticalAlignment": "middle",
      "textColor": "#666666",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": 12,
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": 1,
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": 2,
      "paddingTop": 2,
      "paddingRight": 2,
      "paddingBottom": 2,
      "printIf": "",
      "removeEmptyElement": false,
      "alwaysPrintOnSamePage": true,
      "pattern": "",
      "link": "",
      "cs_condition": "",
      "cs_styleId": "",
      "cs_bold": false,
      "cs_italic": false,
      "cs_underline": false,
      "cs_strikethrough": false,
      "cs_horizontalAlignment": "left",
      "cs_verticalAlignment": "top",
      "cs_textColor": "#000000",
      "cs_backgroundColor": "",
      "cs_font": "helvetica",
      "cs_fontSize": 12,
      "cs_lineSpacing": 1,
      "cs_borderColor": "#000000",
      "cs_borderWidth": "1",
      "cs_borderAll": false,
      "cs_borderLeft": false,
      "cs_borderTop": false,
      "cs_borderRight": false,
      "cs_borderBottom": false,
      "cs_paddingLeft": 2,
      "cs_paddingTop": 2,
      "cs_paddingRight": 2,
      "cs_paddingBottom": 2,
      "spreadsheet_hide": false,
      "spreadsheet_column": "",
      "spreadsheet_colspan": "",
      "spreadsheet_addEmptyRow": false,
      "spreadsheet_textWrap": false
    },
    {
      "elementType": "text",
      "id": 8,
      "containerId": "0_footer",
      "x": 0,
      "y": 0,
      "width": 290,
      "height": 30,
      "content": "Created on ${current_date}",
      "richText": false,
      "richTextContent": null,
      "richTextHtml": "",
      "eval": false,
      "styleId": "",
      "bold": false,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "left",
      "verticalAlignment": "middle",
      "textColor": "#666666",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": 12,
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": 1,
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": 2,
      "paddingTop": 2,
      "paddingRight": 2,
      "paddingBottom": 2,
      "printIf": "",
      "removeEmptyElement": false,
      "alwaysPrintOnSamePage": true,
      "pattern": "",
      "link": "",
      "cs_condition": "",
      "cs_styleId": "",
      "cs_bold": false,
      "cs_italic": false,
      "cs_underline": false,
      "cs_strikethrough": false,
      "cs_horizontalAlignment": "left",
      "cs_verticalAlignment": "top",
      "cs_textColor": "#000000",
      "cs_backgroundColor": "",
      "cs_font": "helvetica",
      "cs_fontSize": 12,
      "cs_lineSpacing": 1,
      "cs_borderColor": "#000000",
      "cs_borderWidth": "1",
      "cs_borderAll": false,
      "cs_borderLeft": false,
      "cs_borderTop": false,
      "cs_borderRight": false,
      "cs_borderBottom": false,
      "cs_paddingLeft": 2,
      "cs_paddingTop": 2,
      "cs_paddingRight": 2,
      "cs_paddingBottom": 2,
      "spreadsheet_hide": false,
      "spreadsheet_column": "",
      "spreadsheet_colspan": "",
      "spreadsheet_addEmptyRow": false,
      "spreadsheet_textWrap": false
    }
  ],
  "parameters": [
    {
      "id": 1,
      "name": "page_count",
      "type": "number",
      "arrayItemType": "string",
      "eval": false,
      "nullable": false,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": true,
      "testData": ""
    },
    {
      "id": 2,
      "name": "page_number",
      "type": "number",
      "arrayItemType": "string",
      "eval": false,
      "nullable": false,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": true,
      "testData": ""
    },
    {
      "id": 9,
      "name": "current_date",
      "type": "date",
      "arrayItemType": "string",
      "eval": false,
      "nullable": false,
      "pattern": "d/M/yyyy H:mm",
      "expression": "",
      "showOnlyNameType": false,
      "testData": ""
    },
    {
      "id": 233,
      "name": "data",
      "type": "array",
      "arrayItemType": "string",
      "eval": false,
      "nullable": false,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": false,
      "testData": "",
      "children": [
        {
          "id": 234,
          "name": "row_number",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": false,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": true,
          "testData": ""
        },
        {
          "id": 263,
          "name": "ProdID",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 269,
          "name": "TotalMaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 271,
          "name": "TotalFemaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 270,
          "name": "TotalOtherInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 272,
          "name": "ProductCode",
          "type": "string",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 273,
          "name": "NewMaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 274,
          "name": "NewFemaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 275,
          "name": "NewOtherInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 277,
          "name": "SuspendedMaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 278,
          "name": "SuspendedFemaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 279,
          "name": "SuspendedOtherInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 280,
          "name": "ExpiredMaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 281,
          "name": "ExpiredFemaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 282,
          "name": "ExpiredOtherInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 286,
          "name": "RenewMaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 287,
          "name": "RenewFemaleInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 288,
          "name": "RenewOtherInsurees",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 295,
          "name": "TotalPremiumPaid",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "#,##0.00",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        },
        {
          "id": 351,
          "name": "AvailablePremium",
          "type": "number",
          "arrayItemType": "string",
          "eval": false,
          "nullable": true,
          "pattern": "",
          "expression": "",
          "showOnlyNameType": false,
          "testData": ""
        }
      ]
    },
    {
      "id": 265,
      "name": "yearMonth",
      "type": "string",
      "arrayItemType": "string",
      "eval": false,
      "nullable": true,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": false,
      "testData": "2019-08"
    },
    {
      "id": 266,
      "name": "productCode",
      "type": "string",
      "arrayItemType": "string",
      "eval": false,
      "nullable": true,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": false,
      "testData": ""
    },
    {
      "id": 267,
      "name": "productName",
      "type": "string",
      "arrayItemType": "string",
      "eval": false,
      "nullable": true,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": false,
      "testData": ""
    },
    {
      "id": 268,
      "name": "LocationName",
      "type": "string",
      "arrayItemType": "string",
      "eval": false,
      "nullable": true,
      "pattern": "",
      "expression": "",
      "showOnlyNameType": false,
      "testData": ""
    }
  ],
  "styles": [
    {
      "id": 33,
      "name": "Table Header",
      "bold": true,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "left",
      "verticalAlignment": "middle",
      "textColor": "#000000",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": "10",
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": "1",
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": "2",
      "paddingTop": "2",
      "paddingRight": "2",
      "paddingBottom": "2"
    },
    {
      "id": 34,
      "name": "Table Content",
      "bold": false,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "left",
      "verticalAlignment": "middle",
      "textColor": "#000000",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": "9",
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": "1",
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": "2",
      "paddingTop": "2",
      "paddingRight": "2",
      "paddingBottom": "2"
    },
    {
      "id": 35,
      "name": "Table Content Highlight",
      "bold": true,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "center",
      "verticalAlignment": "middle",
      "textColor": "#3d85c6",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": "9",
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": "1",
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": "2",
      "paddingTop": "2",
      "paddingRight": "2",
      "paddingBottom": "2"
    },
    {
      "id": 350,
      "name": "Table sub-header",
      "bold": true,
      "italic": false,
      "underline": false,
      "strikethrough": false,
      "horizontalAlignment": "center",
      "verticalAlignment": "middle",
      "textColor": "#000000",
      "backgroundColor": "",
      "font": "helvetica",
      "fontSize": "10",
      "lineSpacing": 1,
      "borderColor": "#000000",
      "borderWidth": "1",
      "borderAll": false,
      "borderLeft": false,
      "borderTop": false,
      "borderRight": false,
      "borderBottom": false,
      "paddingLeft": "",
      "paddingTop": "",
      "paddingRight": "",
      "paddingBottom": ""
    }
  ],
  "version": 3,
  "documentProperties": {
    "pageFormat": "A4",
    "pageWidth": "",
    "pageHeight": "",
    "unit": "mm",
    "orientation": "landscape",
    "contentHeight": "",
    "marginLeft": "10",
    "marginTop": "10",
    "marginRight": "10",
    "marginBottom": "10",
    "header": true,
    "headerSize": "50",
    "headerDisplay": "always",
    "footer": true,
    "footerSize": "30",
    "footerDisplay": "always",
    "patternLocale": "en",
    "patternCurrencySymbol": "$"
  }
}
"""

# the udfAvailablePremium function was written before GREATEST() was available in MS SQL and
# availablePremium could be simplified to:
available_premium_expression = """
CASE
WHEN MONTH(DATEADD_DAY(-1,PL."ExpiryDate")) = MONTH(%(LastDay)s) AND YEAR(DATEADD_DAY(-1,PL."ExpiryDate")) = YEAR(%(LastDay)s) AND extract(DAY from (PL."ExpiryDate")) > 1
    THEN GREATEST(1, DATEDIFF_DAY(GREATEST(PR."PayDate", %(LastDay)s),PL."ExpiryDate"))
      * ((PR."Amount")
         /GREATEST(1, DATEDIFF_DAY(GREATEST(PR."PayDate", PL."EffectiveDate"),PL."ExpiryDate")))
WHEN MONTH(GREATEST(PR."PayDate", PL."EffectiveDate")) = MONTH(%(LastDay)s) AND YEAR(GREATEST(PR."PayDate", PL."EffectiveDate")) = YEAR(%(LastDay)s)
    THEN ((%(DaysInMonth)s + 1 - extract (DAY FROM GREATEST(PR."PayDate", PL."EffectiveDate")))
              * (PR."Amount"
                     /GREATEST(1, DATEDIFF_DAY(GREATEST(PR."PayDate", PL."EffectiveDate"),PL."ExpiryDate"))))
WHEN PL."EffectiveDate" < %(LastDay)s AND PL."ExpiryDate" > %(LastDay)s AND PR."PayDate" < %(LastDay)s
    THEN %(DaysInMonth)s
             * (PR."Amount"
                    /GREATEST(1, DATEDIFF_DAY(GREATEST(PR."PayDate", PL."EffectiveDate"),DATEADD_DAY(-1,PL."ExpiryDate"))))
END
"""

policies_primary_indicators_sql = f"""
    SELECT prod."ProdID" as "ProdID", prod."ProductCode" as "ProductCode",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'M'     
            AND pl."PolicyStatus" > 1
            AND pl."EffectiveDate" <= %(LastDay)s
            AND pl."ExpiryDate" > %(LastDay)s
        ) AS "TotalMaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'F'     
            AND pl."PolicyStatus" > 1
            AND pl."EffectiveDate" <= %(LastDay)s
            AND pl."ExpiryDate" > %(LastDay)s
        ) AS "TotalFemaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'O'     
            AND pl."PolicyStatus" > 1
            AND pl."EffectiveDate" <= %(LastDay)s
            AND pl."ExpiryDate" > %(LastDay)s
        ) AS "TotalOtherInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'M'     
            AND pl."PolicyStatus" > 1
            AND pl."PolicyStage" = 'N'
            AND YEAR(pl."EffectiveDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."EffectiveDate") = MONTH(%(LastDay)s)
        ) AS "NewMaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'F'     
            AND pl."PolicyStatus" > 1
            AND pl."PolicyStage" = 'N'
            AND YEAR(pl."EffectiveDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."EffectiveDate") = MONTH(%(LastDay)s)
        ) AS "NewFemaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'O'     
            AND pl."PolicyStatus" > 1
            AND pl."PolicyStage" = 'N'
            AND YEAR(pl."EffectiveDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."EffectiveDate") = MONTH(%(LastDay)s)
        ) AS "NewOtherInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'M'     
            AND pl."PolicyStatus" > 1
            AND pl."PolicyStage" = 'R'
            AND YEAR(pl."EnrollDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."EnrollDate") = MONTH(%(LastDay)s)
        ) AS "RenewMaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'F'     
            AND pl."PolicyStatus" > 1
            AND pl."PolicyStage" = 'R'
            AND YEAR(pl."EnrollDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."EnrollDate") = MONTH(%(LastDay)s)
        ) AS "RenewFemaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'O'     
            AND pl."PolicyStatus" > 1
            AND pl."PolicyStage" = 'R'
            AND YEAR(pl."EnrollDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."EnrollDate") = MONTH(%(LastDay)s)
        ) AS "RenewOtherInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'M'     
            AND pl."PolicyStatus" = 4
            AND YEAR(pl."ValidityFrom") = YEAR(%(LastDay)s)
            AND MONTH(pl."ValidityFrom") = MONTH(%(LastDay)s)
        ) AS "SuspendedMaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'F'     
            AND pl."PolicyStatus" = 4
            AND YEAR(pl."ValidityFrom") = YEAR(%(LastDay)s)
            AND MONTH(pl."ValidityFrom") = MONTH(%(LastDay)s)
        ) AS "SuspendedFemaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'O'     
            AND pl."PolicyStatus" = 4
            AND YEAR(pl."ValidityFrom") = YEAR(%(LastDay)s)
            AND MONTH(pl."ValidityFrom") = MONTH(%(LastDay)s)
        ) AS "SuspendedOtherInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'M'     
            AND pl."PolicyStatus" > 1
            AND YEAR(pl."ExpiryDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."ExpiryDate") = MONTH(%(LastDay)s)
        ) AS "ExpiredMaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'F'     
            AND pl."PolicyStatus" > 1
            AND YEAR(pl."ExpiryDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."ExpiryDate") = MONTH(%(LastDay)s)
        ) AS "ExpiredFemaleInsurees",
        COUNT(ins."InsureeID") FILTER (
            WHERE Ins."Gender" = 'O'     
            AND pl."PolicyStatus" > 1
            AND YEAR(pl."ExpiryDate") = YEAR(%(LastDay)s)
            AND MONTH(pl."ExpiryDate") = MONTH(%(LastDay)s)
        ) AS "ExpiredOtherInsurees",
        sum(pr."Amount") FILTER (
            WHERE YEAR(pr."PayDate") = YEAR(%(LastDay)s)
            AND MONTH(pr."PayDate") = MONTH(%(LastDay)s)
        ) AS "TotalPremiumPaid",
        SUM({available_premium_expression}) AS "AvailablePremium"
    FROM "tblPolicy" pl
    INNER JOIN "tblProduct" prod ON prod."ProdID" = pl."ProdID"
    INNER JOIN "tblFamilies" fam ON fam."FamilyID" = pl."FamilyID"
    INNER JOIN "tblInsuree" ins ON ins."InsureeID" = fam."InsureeID"
    INNER JOIN "tblLocations" v ON v."LocationId" = fam."LocationId"
    INNER JOIN "tblLocations" w ON w."LocationId" = v."ParentLocationId"
    INNER JOIN "tblLocations" d ON d."LocationId" = w."ParentLocationId"
    INNER JOIN "tblLocations" r ON r."LocationId" = d."ParentLocationId"
    LEFT JOIN "tblPremium" pr ON pr."PolicyID" = pl."PolicyID"
    WHERE pl."ValidityTo" IS NULL
    AND prod."ValidityTo" IS NULL
    AND fam."ValidityTo" IS NULL
    AND ins."ValidityTo" IS NULL
    and pr."ValidityTo" IS NULL
    AND (prod."ProdID" = %(ProdId)s OR %(ProdId)s = 0)
    AND (r."LocationId" = %(LocationId)s OR d."LocationId" = %(LocationId)s OR COALESCE(%(LocationId)s, 0) = 0 OR %(LocationId)s = 0)
    GROUP BY prod."ProdID", prod."ProductCode";
"""


def policies_primary_indicators_query(user, yearMonth, locationId=0, prodId=0, **kwargs):
    first_day = datetime.strptime(yearMonth, "%Y-%m-%d").replace(day=1)
    last_day = first_day + relativedelta(months=1) - timedelta(days=1)
    days_in_month = last_day.day

    with connection.cursor() as cur:
        try:
            cur.execute(
                policies_primary_indicators_sql,
                {
                    "LastDay": last_day,
                    "LocationId": locationId,
                    "ProdId": prodId,
                    "DaysInMonth": days_in_month,
                },
            )
            return {
                "data": dictfetchall(cur)
            }
        except Exception as e:
            logger.exception("Error fetching policies primary indicators query")
            raise e

    logger.error("Policies primary indicators query arrived at end of function")
    return {"data": None}
